<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Helper class with public methods to access the license API:
 * check()
 * activate()
 * deactivate()
 *
 * @author RavanH
 * @version 1.2
 */
class EDD_SL_License {

	/*************
	* PROPERTIES *
	*************/

	/**
	 * Remote post response data.
	 *
	 * $license_data->success : bolean
	 * $license_data->license :
	 * -- 'valid' -> Your license key was succesfully activated for this site.
	 * -- 'inactive' -> Your license is not active for this site.
	 * -- 'deactivated' -> Your license key was succesfully deactivated for this site.
	 * -- 'expired' -> Your license key has expired on %s. + formated expired date
	 * -- 'invalid' -> May return this on trying to (de)activate an expired/revoked license, check error property for more info
	 * -- 'invalid_item_id' -> When check is done for a key that does not match the product id
	 * -- 'failed' -> General validation error, may return on trying to (de)activate an empty or incorrect key
	 * $license_data->error :
	 * -- 'expired' -> Your license key has expired on %s. + formated expired date
	 * -- 'disabled' | 'revoked' -> Your license key has been disabled.
	 * -- 'invalid' | 'missing' -> This appears to be an invalid license key.
	 * -- 'invalid_item_id' -> This appears to be an invalid license key for %s. + package name
	 * -- 'site_inactive' -> Your license is not active for this URL.
	 * -- 'item_name_mismatch' -> This appears to be an invalid license key for %s. + package name
	 * -- 'no_activations_left' -> Your license key has reached its activation limit.
	 * -- 'error' -> An error occurred, please try again.
	 * $license_data->expires :
	 * -- timestamp, make human readable with
	 *    date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
	 * $license_data->license_limit :
	 * -- integer, max number of sites for this license
	 * $license_data->activations_left :
	 * -- integer, remaining sites to activate for this license
	 * $license_data->payment_id :
	 * -- integer, can be used to create a link to license management page like :
	 *    trailingslashit( $store_url ) . 'account/?action=manage_licenses&payment_id=' . $license_data->payment_id
	 * $license_data->item_id : ( same as input $item_id )
	 * -- integer, can be used to create a link to license renewal checkout page like :
	 *    trailingslashit( $store_url ) . 'checkout/?edd_license_key=' . $key . '&download_id=' . $license_data->item_id
	 *
	 * @property object|null
	 */
	public $license_data;

	/**
	 * License data.
	 */

	/**
	 * License status for this site, reduced from property $license_data->license
	 * Can be used to set coherent admin status messages.
	 *
	 * Possible values: 'valid', 'inactive', 'expired' or 'invalid'
	 *
	 * @property string $status
	 */
	public $status = '';

	/**
	 * Error, deduced from $license_data->license and $license_data->error responses
	 * Can be used to set coherent admin error message.
	 *
	 * Possible values: see $license_data->error
	 *
	 * @property string $error
	 */
	public $error = '';

	/**
	 * Expiration date, populated with the property $license_data->expires
	 * Note: ONLY set when the license is for current product ID,
	 *       NOT when the license is for another product (even thought such a response passes expiration data) !
	 *
	 * A timestamp, make human readable with
	 * date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
	 *
	 * @property string $expires
	 */
	public $expires = '';

	/**
	 * Payement ID, populated with the property $license_data->payment_id
	 * Note: Only set when the license is valid and for current product ID,
	 *       NOT when the license is for another product (even thought such a response passes a payment_id) !
	 *
	 * Can be used to create link to account license management page:
	 * trailingslashit( $store_url ) . 'account/?action=manage_licenses&payment_id=' . $license_data->payment_id
	 *
	 * @property string $payment_id
	 */
	public $payment_id = 0;

	private $key = '';
	private $store_url = '';
	private $item_id = '';
	private $sslverify = false;
	private $timeout = 15;

	/**********
	* METHODS *
	**********/

	/**
	 * Class constructor.
	 *
	 * @uses trailingslashit()
	 *
	 * @param string  $_key       The license key.
	 * @param string  $_store_url Download store URL.
	 * @param int     $_item_id   Download item ID.
	 */
	function __construct( $_key = '', $_store_url = '', $_item_id = 0 )
	{
		// populate
		$this->key = trim( $_key );
		$this->store_url = trailingslashit( $_store_url );
		$this->item_id = urlencode( (int) $_item_id );
	}

	/**
	 * Check the license key.
	 *
	 * Populates public property $license_data with the
	 * json response object (or null) if one is received and returns:
	 * -- true on valid license
	 * -- false on invalid license (check $this->license_data->error for error code)
	 * -- response status code (other than 200) on communication failure
	 * -- WP_Error object if encountered.
	 *
	 * @since 1.0
	 *
	 * @uses $this->call(), is_object(), property_exists()
	 *
	 * @return bool|int|object
	 */
	public function check()
	{
		// Call the custom API with action check_license
		$response = $this->call( 'check_license' );

		if ( true === $response ) {
			if( is_object($this->license_data)
			&& property_exists($this->license_data, 'success')
			&& $this->license_data->success === true ) {
				return true;
			} else {
				return false;
			}
		}

		return $response;
	}

	/**
	 * Deactivate the license key.
	 *
	 * Populates public property $license_data with the
	 * json response object (or null) if one is received and returns:
	 * -- true on success
	 * -- false on failure (check $this->license_data->error for deactivation error)
	 * -- response status code (other than 200) on communication failure
	 * -- WP_Error object if encountered.
	 *
	 * @since 1.0
	 *
	 * @uses $this->call()
	 *
	 * @return bool|int|object
	 */
	public function deactivate()
	{
		// Call the custom API with action deactivate_license
		$response = $this->call( 'deactivate_license' );

		if ( true === $response ) {
			if ( is_object($this->license_data)
			&& property_exists($this->license_data, 'success')
			&& true === $this->license_data->success ) {
				return true;
			} else {
				return false;
			}
		}

		return $response;
	}

	/**
	 * Activate the license key.
	 *
	 * Populates public property $license_data with the
	 * json response object (or null) if one is received and returns:
	 * -- true on success
	 * -- false on failure (check $this->license_data->error for deactivation error)
	 * -- response status code (other than 200) on communication failure
	 * -- WP_Error object if encountered.
	 *
	 * @since 1.0
	 *
	 * @uses $this->call()
	 *
	 * @return bool|int|object
	 */
	public function activate()
	{
		// Call the custom API with action deactivate_license
		$response = $this->call( 'activate_license' );

		if ( true === $response ) {
			if ( is_object($this->license_data)
			&& property_exists($this->license_data, 'success')
			&& true === $this->license_data->success ) {
				return true;
			} else {
				return false;
			}
		}

		return $response;
	}

	/**
	 * Call home with an action.
	 *
	 * Accepts an action value, populating public property $license_data with the
	 * json response object (or null) if one is received and returns:
	 * -- true on success
	 * -- response status code (other than 200) on communication failure
	 * -- WP_Error object if encountered.
	 *
	 * @since 1.0
	 *
	 * @uses wp_remote_post(), wp_remote_retrieve_response_code(), json_decode(), wp_remote_retrieve_body()
	 *
	 * @param var $action The API action.
	 * @return bool|int|object true, response status code or WP_Error
	 */
	private function call( $_action )
	{

		if ( $this->store_url == trailingslashit ( home_url() ) ) {
			return 999; // Don't allow a plugin to ping itself
		}

		// data to send in our API request
		$api_params = array(
			'edd_action' => $_action,
			'license'    => $this->key,
			'item_id'    => $this->item_id,
			'url'        => home_url()
		);

		// Call the custom API.
		$response = wp_remote_post(
			$this->store_url,
			array(
				'timeout' => $this->timeout,
				'sslverify' => (bool) apply_filters( 'edd_sl_api_request_verify_ssl', $this->sslverify, $this ),
				'body' => $api_params
			)
		);

		if ( is_wp_error( $response ) ) {
			return $response;
		}

		$response_code = wp_remote_retrieve_response_code( $response );

		if ( 200 !== $response_code ) {
			return $response_code;
		}

		$this->license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// process license data and populate properties we can find
		if ( is_object( $this->license_data ) ) :

			// last response license status, varies depending on last call action
			switch ( $this->license_data->license ) {
				case 'valid':
				case 'inactive':
				// license status
				$this->status = $this->license_data->license;
				// license payment id
				$this->payment_id = property_exists( $this->license_data, 'payment_id' ) && ! empty( $this->license_data->payment_id ) ? $this->license_data->payment_id : $this->payment_id;
				// license expiration date
				$this->expires = property_exists( $this->license_data, 'expires' ) && ! empty ( $this->license_data->expires ) ? $this->license_data->expires : $this->expires;
			break;

				case 'expired':
				// license status
				$this->status = $this->license_data->license;
				// license payment id
				$this->payment_id = property_exists( $this->license_data, 'payment_id' ) && ! empty( $this->license_data->payment_id ) ? $this->license_data->payment_id : $this->payment_id;
				// response error
				$this->error = property_exists( $this->license_data, 'error' ) && ! empty( $this->license_data->error ) ? $this->license_data->error : $this->license_data->license;
				// license expiration date
				$this->expires = property_exists( $this->license_data, 'expires' ) && ! empty ( $this->license_data->expires ) ? $this->license_data->expires : $this->expires;
				break;

				case 'deactivated':
				case 'site_inactive':
				// license status
				$this->status = 'inactive';
				// license payment id
				$this->payment_id = property_exists( $this->license_data, 'payment_id' ) && ! empty( $this->license_data->payment_id ) ? $this->license_data->payment_id : $this->payment_id;
				// license expiration date
				$this->expires = property_exists( $this->license_data, 'expires' ) && ! empty ( $this->license_data->expires ) ? $this->license_data->expires : $this->expires;
				break;

				case 'invalid':
				case 'invalid_item_id':
				// license status
				$this->status = 'invalid';
				// response error
				$this->error = property_exists( $this->license_data, 'error' ) && ! empty( $this->license_data->error ) ? $this->license_data->error : $this->license_data->license;
				// don't set expiration or payment id even if that data was passed by remote post response
				break;

				case 'failed':
				// Leave status unknown
				// response error or 'invalid'
				$this->error = property_exists( $this->license_data, 'error' ) && ! empty( $this->license_data->error ) ? $this->license_data->error : 'invalid';
				break;

				default:
				// Leave status unknown
				// response error or '' (empty)
				$this->error = property_exists( $this->license_data, 'error' ) && ! empty( $this->license_data->error ) ? $this->license_data->error : '';
			}

		endif;

		if ( defined('WP_DEBUG') && WP_DEBUG ) {
			// Debugging prints license data object
			error_log( print_r( $this->license_data, true ) );
		}

		return true;
	}

}
